clear all
close all

% A good way to test if you gradient is implemented correct
% is to compare the analytical and numerical gradient at 
% ranodm points. E.g.:
%
% X0=rand(2,1)*3-1.5;
% [f,J]=Rosen(X0);
% numJ=zeros(size(J));
% delta=1e-7;
% for i=1:size(J,2),
%     dX0=X0; dX0(i)=dX0(i)+delta;
%     numJ(:,i)=(Rosen(dX0)-f)/delta;
% end
% norm(J-numJ)/norm(J+numJ)
% return

x=-1.5:0.01:1.5;
y=-1:0.01:1.5;
f=zeros(length(x),length(y));
X=f;
Y=f;
for i=1:length(x),
    for j=1:length(y),
        q=Rosen([x(i);y(j)]);
        f(i,j)=q'*q;
        X(i,j)=x(i);
        Y(i,j)=y(j);
    end
end

contour(X,Y,f,logspace(-1,3,15))
axis image

[X0, info,perf] = marquardt('Rosen', [-1.2;1]);
hold on
h=plot(X0(1,:),X0(2,:),'r*-')
set(h,'LineWidth',1)
hold off

ValidSteps=[X0(:,1)];
for i=2:size(X0,2),
    if(perf(1,i)<perf(1,i-1))
        ValidSteps=[ValidSteps X0(:,i)];
    end
end

hold on
plot(ValidSteps(1,:),ValidSteps(2,:),'ko:')
hold off

